@@ -158,6 +158,11 @@ def arctanh(a: Numeric):
     return anp.arctanh(a)
 
 
+@dispatch
+def loggamma(a: Numeric):
+    return asps.gammaln(a)
+
+
 @dispatch
 def erf(a: Numeric):
     return asps.erf(a)
