@@ -21,9 +21,9 @@ def choice(
     # Feeding `a` to `choice` will not work if `a` is higher-dimensional.
     inds = state.choice(a.shape[0], n, replace=True, p=p)
     choices = a[inds]
-    return state, choices[0] if n == 1 else choices
+    return state, choices
 
 
 @dispatch
-def choice(a: AGNumeric, n: Int, *, p: Union[AGNumeric, None] = None):
-    return choice(np.random.random.__self__, a, n, p=p)[1]
+def choice(a: AGNumeric, *shape: Int, p: Union[AGNumeric, None] = None):
+    return choice(np.random.random.__self__, a, *shape, p=p)[1]
