@@ -183,6 +183,11 @@ def arctanh(a: Numeric):
     return np.arctanh(a)
 
 
+@dispatch
+def loggamma(a: Numeric):
+    return sps.gammaln(a)
+
+
 @dispatch
 def erf(a: Numeric):
     return sps.erf(a)
