@@ -1,4 +1,6 @@
 import sys
+from functools import reduce
+from operator import mul
 
 import numpy as np
 from plum.type import VarArgs, Union
@@ -17,6 +19,8 @@ __all__ = [
     "choice",
     "randint",
     "randperm",
+    "randgamma",
+    "randbeta",
 ]
 
 
@@ -87,7 +91,8 @@ def set_global_random_state(state: RandomState):
 
     NOTE:
         In TensorFlow, setting the global random state does NOT fix the randomness
-        for non-LAB random calls, like `tf.random.normal`. Use `B.set_seed` instead!
+        for non-LAB random calls, like `tf.random.normal`. Use `B.set_random_seed`
+        instead!
 
     Args:
         state (random state): Random state to set.
@@ -101,14 +106,16 @@ def global_random_state(a):
 
 @dispatch
 @abstract()
-def rand(dtype: DType, *shape: Int):  # pragma: no cover
+def rand(state: RandomState, dtype: DType, *shape: Int):  # pragma: no cover
     """Construct a U[0, 1] random tensor.
 
     Args:
+        state (random state, optional): Random state.
         dtype (dtype, optional): Data type. Defaults to the default data type.
-        *shape (shape, optional): Shape of the tensor. Defaults to `()`.
+        *shape (shape, optional): Shape of the sample. Defaults to `()`.
 
     Returns:
+        state (random state, optional): Random state.
         tensor: Random tensor.
     """
 
@@ -136,9 +143,10 @@ def randn(state: RandomState, dtype: DType, *shape: Int):  # pragma: no cover
     Args:
         state (random state, optional): Random state.
         dtype (dtype, optional): Data type. Defaults to the default data type.
-        *shape (shape, optional): Shape of the tensor. Defaults to `()`.
+        *shape (shape, optional): Shape of the sample. Defaults to `()`.
 
     Returns:
+        state (random state, optional): Random state.
         tensor: Random tensor.
     """
 
@@ -159,44 +167,27 @@ def randn(ref: Numeric):
 
 
 @dispatch
-@abstract()
 def choice(
     state: RandomState,
     a: Numeric,
-    n: Int,
-    *,
+    *shape: Int,
     p: Union[Numeric, None] = None,
-):  # pragma: no cover
+):
     """Randomly choose from a tensor *with* replacement.
 
     Args:
         state (random state, optional): Random state.
         a (tensor): Tensor to choose from.
-        n (int, optional): Number of samples. Defaults to `1`.
-        p (tensor, optional): Probabilities to sample with.
+        *shape (int): Shape of the sample. Defaults to `()`.
+        p (vector, optional): Probabilities to sample with.
 
     Returns:
+        state (random state, optional): Random state.
         tensor: Choices.
     """
-
-
-@dispatch
-def choice(
-    state: RandomState,
-    a: Numeric,
-    *,
-    p: Union[Numeric, None] = None,
-):
-    return choice(state, a, 1, p=p)
-
-
-@dispatch
-def choice(
-    a: Numeric,
-    *,
-    p: Union[Numeric, None] = None,
-):
-    return choice(a, 1, p=p)
+    n = reduce(mul, shape, 1)
+    state, choices = choice(state, a, n, p=p)
+    return state, B.reshape(choices, *shape, *B.shape(choices)[1:])
 
 
 @dispatch
@@ -218,6 +209,7 @@ def randint(
         upper (int): Upper bound. Must be given as a keyword argument.
 
     Returns:
+        state (random state, optional): Random state.
         tensor: Random tensor.
     """
 
@@ -248,6 +240,7 @@ def randperm(state: RandomState, dtype: DType, n: Int):  # pragma: no cover
         n (int): Length of the permutation.
 
     Returns:
+        state (random state, optional): Random state.
         tensor: Random permutation.
     """
 
@@ -255,3 +248,96 @@ def randperm(state: RandomState, dtype: DType, n: Int):  # pragma: no cover
 @dispatch
 def randperm(n: Int):
     return randperm(B.default_dtype, n)
+
+
+@dispatch
+@abstract()
+def randgamma(
+    state: RandomState,
+    dtype: DType,
+    *shape: Int,
+    alpha: Numeric,
+    scale: Numeric,
+):  # pragma: no cover
+    """Construct a tensor of gamma random variables with shape parameter `alpha` and
+        scale `scale`.
+
+    Args:
+        state (random state, optional): Random state.
+        dtype (dtype, optional): Data type. Defaults to the default data type.
+        *shape (shape, optional): Shape of the tensor. Defaults to `()`.
+        alpha (scalar): Shape parameter.
+        scale (scalar): Scale parameter.
+
+    Returns:
+        state (random state, optional): Random state.
+        tensor: Random tensor.
+    """
+
+
+@dispatch.multi((Int,), (VarArgs(Int),))  # Single integer is a not a reference.
+def randgamma(*shape: Int, alpha: Numeric, scale: Numeric):
+    return randgamma(B.default_dtype, *shape, alpha=alpha, scale=scale)
+
+
+@dispatch
+def randgamma(state: RandomState, ref: Numeric, *, alpha: Numeric, scale: Numeric):
+    return randgamma(state, B.dtype(ref), *B.shape(ref), alpha=alpha, scale=scale)
+
+
+@dispatch
+def randgamma(ref: Numeric, *, alpha: Numeric, scale: Numeric):
+    return randgamma(B.dtype(ref), *B.shape(ref), alpha=alpha, scale=scale)
+
+
+@dispatch
+def randbeta(
+    state: RandomState,
+    dtype: DType,
+    *shape: Int,
+    alpha: Numeric,
+    beta: Numeric,
+):
+    """Construct a tensor of beta random variables with shape parameters `alpha` and
+        `beta`.
+
+    Args:
+        state (random state, optional): Random state.
+        dtype (dtype, optional): Data type. Defaults to the default data type.
+        *shape (shape, optional): Shape of the tensor. Defaults to `()`.
+        alpha (scalar): Shape parameter `alpha`.
+        beta (scalar): Shape parameter `beta`.
+
+    Returns:
+        state (random state, optional): Random state.
+        tensor: Random tensor.
+    """
+    state, x = randgamma(state, dtype, *shape, alpha=alpha, scale=1)
+    state, y = randgamma(state, dtype, *shape, alpha=beta, scale=1)
+    return state, x / (x + y)
+
+
+@dispatch
+def randbeta(dtype: DType, *shape: Int, alpha: Numeric, beta: Numeric):
+    return randbeta(
+        B.global_random_state(dtype),
+        dtype,
+        *shape,
+        alpha=alpha,
+        beta=beta,
+    )[1]
+
+
+@dispatch.multi((Int,), (VarArgs(Int),))  # Single integer is a not a reference.
+def randbeta(*shape: Int, alpha: Numeric, beta: Numeric):
+    return randbeta(B.default_dtype, *shape, alpha=alpha, beta=beta)
+
+
+@dispatch
+def randbeta(state: RandomState, ref: Numeric, *, alpha: Numeric, beta: Numeric):
+    return randbeta(state, B.dtype(ref), *B.shape(ref), alpha=alpha, beta=beta)
+
+
+@dispatch
+def randbeta(ref: Numeric, *, alpha: Numeric, beta: Numeric):
+    return randbeta(B.dtype(ref), *B.shape(ref), alpha=alpha, beta=beta)
