@@ -1,7 +1,7 @@
 import torch
 from plum import Union
 
-from . import B, dispatch
+from . import dispatch, B, Numeric
 from ..types import TorchNumeric, TorchDType, Int, TorchRandomState
 
 __all__ = []
@@ -81,12 +81,12 @@ def choice(
     if p is None:
         p = torch.ones(a.shape[0], dtype=B.dtype_float(a), device=a.device)
     choices = a[torch.multinomial(p, n, replacement=True, generator=state)]
-    return state, choices[0] if n == 1 else choices
+    return state, choices
 
 
 @dispatch
-def choice(a: TorchNumeric, n: Int, *, p: Union[TorchNumeric, None] = None):
-    return choice(global_random_state(a), a, n, p=p)[1]
+def choice(a: TorchNumeric, *shape: Int, p: Union[TorchNumeric, None] = None):
+    return choice(global_random_state(a), a, *shape, p=p)[1]
 
 
 @dispatch
@@ -128,3 +128,23 @@ def randperm(state: TorchRandomState, dtype: TorchDType, n: Int):
 @dispatch
 def randperm(dtype: TorchDType, n: Int):
     return randperm(global_random_state(dtype), dtype, n)[1]
+
+
+@dispatch
+def randgamma(
+    state: TorchRandomState,
+    dtype: TorchDType,
+    *shape: Int,
+    alpha: Numeric,
+    scale: Numeric,
+):
+    alpha = B.to_active_device(B.cast(dtype, alpha))
+    scale = B.to_active_device(B.cast(dtype, scale))
+    alpha = B.broadcast_to(alpha, *shape)
+    return state, torch._standard_gamma(alpha, generator=state) * scale
+
+
+@dispatch
+def randgamma(dtype: TorchDType, *shape: Int, alpha: Numeric, scale: Numeric):
+    state = global_random_state(dtype)
+    return randgamma(state, dtype, *shape, alpha=alpha, scale=scale)[1]
