@@ -71,6 +71,8 @@ def test_create_random_state(dtype):
         (B.randn, lambda x: x, False),
         (lambda *args: B.randint(*args, lower=0, upper=10), B.dtype_int, False),
         (B.randperm, B.dtype_int, True),
+        (lambda *args: B.randgamma(*args, alpha=0.5, scale=0.5), lambda x: x, False),
+        (lambda *args: B.randbeta(*args, alpha=0.5, beta=0.5), lambda x: x, False),
     ],
 )
 @pytest.mark.parametrize("t", [np.float32, tf.float32, torch.float32, jnp.float32])
@@ -137,6 +139,17 @@ def test_randint_bounds(t, check_lazy_shapes):
     assert B.randint(t, lower=10, upper=11) == 10
 
 
+@pytest.mark.parametrize("t", [np.float32, tf.float32, torch.float32, jnp.float32])
+def test_randgamma_parameters(t, check_lazy_shapes):
+    approx(B.randgamma(t, alpha=1, scale=0), 0, atol=1e-6)
+
+
+@pytest.mark.parametrize("t", [np.float32, tf.float32, torch.float32, jnp.float32])
+def test_randbeta_parameters(t, check_lazy_shapes):
+    approx(B.randbeta(t, alpha=1e-6, beta=1), 0, atol=1e-6)
+    approx(B.randbeta(t, alpha=1, beta=1e-6), 1, atol=1e-6)
+
+
 def test_torch_global_random_state(mocker):
     # Check CPU specifications.
     B.ActiveDevice.active_name = None
@@ -208,15 +221,13 @@ def test_choice(x, p, check_lazy_shapes):
 
     # Check shape.
     assert B.shape(B.choice(x, **p)) == B.shape(x)[1:]
-    assert B.shape(B.choice(x, 1, **p)) == B.shape(x)[1:]
-    assert B.shape(B.choice(x, 5, **p))[0] == 5
-    assert B.shape(B.choice(x, 5, **p))[1:] == B.shape(x)[1:]
+    assert B.shape(B.choice(x, 5, **p)) == (5,) + B.shape(x)[1:]
+    assert B.shape(B.choice(x, 5, 5, **p)) == (5, 5) + B.shape(x)[1:]
 
     assert isinstance(B.choice(state, x, **p)[0], B.RandomState)
     assert B.shape(B.choice(state, x, **p)[1]) == B.shape(x)[1:]
-    assert B.shape(B.choice(state, x, 1, **p)[1]) == B.shape(x)[1:]
-    assert B.shape(B.choice(state, x, 5, **p)[1])[0] == 5
-    assert B.shape(B.choice(state, x, 5, **p)[1])[1:] == B.shape(x)[1:]
+    assert B.shape(B.choice(state, x, 5, **p)[1]) == (5,) + B.shape(x)[1:]
+    assert B.shape(B.choice(state, x, 5, 5, **p)[1]) == (5, 5) + B.shape(x)[1:]
 
     # Check correctness.
     dtype = B.dtype(x)
